﻿//This C# code file was released under the Ms-PL license
//http://www.opensource.org/licenses/ms-pl.html
//This script was originally intended for use with Microsoft Dynamics AX
//and maintained and distributed as a project on CodePlex
//http://dynamicsaxadmin.codeplex.com

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Common;
using System.Data.SqlClient;

namespace CodeCrib.AX.Sql
{
    public class DbManagement
    {
        private const int DB_TIMEOUT = 600;

        // ALTER DATABASE does not accept a variable for the database name
        // Must use manual replacement in the query string (indicated by {DbName}).
        private const string DB_RESTORE_COMMAND = @"
IF (EXISTS(SELECT 'x' FROM [sys].[databases] WHERE [sys].[databases].[name] = @DbName))
BEGIN
    ALTER DATABASE [{DbName}] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
END
RESTORE DATABASE @DbName FROM  DISK = @FileName WITH  FILE = 1,  NOUNLOAD,  REPLACE,  STATS = 5;
IF (EXISTS(SELECT 'x' FROM [sys].[databases] WHERE [sys].[databases].[name] = @DbName))
BEGIN
    WAITFOR DELAY '00:00:03';
    ALTER DATABASE [{DbName}] SET MULTI_USER;
END";

        public static void RestoreDbFromFile(string serverName, string dbName, string fileName)
        {
            using (DbConnection dbConnection = CreateConnection(serverName))
            {
                // These methods may throw when there is a problem connecting or restoring the
                // database.  In that case, let the exception bubble up to the caller.
                dbConnection.Open();

                DbCommand dbCommand = dbConnection.CreateCommand();
                dbCommand.CommandTimeout = DB_TIMEOUT;
                dbCommand.CommandText = DB_RESTORE_COMMAND;
                dbCommand = AddInParameters(dbCommand, dbName, fileName);

                dbCommand.ExecuteNonQuery();

                dbConnection.Close();
            }
        }

        private static DbCommand AddInParameters(DbCommand dbCommand, string dbName, string fileName)
        {
            // ALTER DATABASE does not accept a variable for the database name
            // Must use manual replacement in the query string.
            dbCommand.CommandText = dbCommand.CommandText.Replace("{DbName}", dbName);

            DbParameter dbParam = dbCommand.CreateParameter();
            dbParam.ParameterName = "@DbName";
            dbParam.Value = dbName;
            dbCommand.Parameters.Add(dbParam);

            DbParameter fileParam = dbCommand.CreateParameter();
            fileParam.ParameterName = "@FileName";
            fileParam.Value = fileName;
            dbCommand.Parameters.Add(fileParam);

            return dbCommand;
        }

        public static DbConnection CreateConnection(string serverName)
        {
            DbConnection dbConnection = SqlClientFactory.Instance.CreateConnection();

            SqlConnectionStringBuilder connectionStringBuilder = new SqlConnectionStringBuilder();

            connectionStringBuilder.IntegratedSecurity = true;
            connectionStringBuilder.DataSource = serverName;

            dbConnection.ConnectionString = connectionStringBuilder.ConnectionString;

            return dbConnection;
        }
    }
}
