﻿//This C# code file was released under the Ms-PL license
//http://www.opensource.org/licenses/ms-pl.html
//This script was originally intended for use with Microsoft Dynamics AX
//and maintained and distributed as a project on CodePlex
//http://dynamicsaxadmin.codeplex.com

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;
using System.IO;

namespace CodeCrib.AX.TFS
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public class GenerateCIL : CodeActivity
    {
        public InArgument<string> ClientExecutablePath { get; set; }

        public InArgument<int> TimeOutMinutes { get; set; }

        public InArgument<string> ConfigurationFile { get; set; }


        protected override void Execute(CodeActivityContext context)
        {
            int timeOutMinutes = TimeOutMinutes.Get(context);
            string clientExePath = ClientExecutablePath.Get(context);
            string configurationFile = ConfigurationFile.Get(context);

            context.TrackBuildMessage("Generating CIL.");

            CodeCrib.AX.Client.Commands.GenerateCIL compile = new CodeCrib.AX.Client.Commands.GenerateCIL()
            {
                Minimize = true,
                LazyClassLoading = true,
                LazyTableLoading = true
            };

            if (!string.IsNullOrEmpty(configurationFile))
            {
                compile.ConfigurationFile = configurationFile;
            }

            if (string.IsNullOrEmpty(clientExePath))
                CodeCrib.AX.Client.Client.ExecuteCommand(compile, timeOutMinutes);
            else
                CodeCrib.AX.Client.Client.ExecuteCommand(clientExePath, compile, timeOutMinutes);

            var serverConfig = Helper.GetServerConfig(configurationFile);

            CodeCrib.AX.Client.CILGenerationOutput output = null;
            try
            {
                output = CodeCrib.AX.Client.CILGenerationOutput.CreateFromFile(string.Format(@"{0}\XppIL\Dynamics.Ax.Application.dll.log", System.Environment.ExpandEnvironmentVariables(serverConfig.AlternateBinDirectory)));
            }
            catch (System.IO.FileNotFoundException)
            {
                throw new Exception("CIL generation log could not be found");
            }
            catch (Exception ex)
            {
                throw new Exception(string.Format("Error parsing CIL generation log: {0}", ex.Message));
            }

            bool hasErrors = false;
            foreach (var item in output.Output)
            {
                string compileMessage;

                if (item.LineNumber > 0)
                    compileMessage = String.Format("Object {0} method {1}, line {2} : {3}", item.ElementName, item.MethodName, item.LineNumber, item.Message);
                else
                    compileMessage = String.Format("Object {0} method {1} : {2}", item.ElementName, item.MethodName, item.Message);

                switch (item.Severity)
                {
                    // Compile Errors
                    case 0:
                        context.TrackBuildError(compileMessage);
                        hasErrors = true;
                        break;
                    // Compile Warnings
                    case 1:
                        context.TrackBuildWarning(compileMessage);
                        break;
                    // "Other"
                    case 4:
                    default:
                        context.TrackBuildMessage(item.Message);
                        break;
                }
            }
            if (hasErrors)
                throw new Exception("CIL error(s) found");
        }
    }
}
